#!/usr/bin/perl -w  -I.


=head1 NAME

Server.pm - A network server offering Xen control servers.

=cut


=head1 NETWORK PROTOCOL

  TODO describe.

=cut


=head1 CONFIGURATION FILE

  TODO describe.

=cut


=head1 AUTHOR


 Steve
 --
 http://www.steve.org.uk/

 $Id: Server.pm,v 1.6 2006/02/20 18:23:24 steve Exp $

=cut



package Xen::Monitor::Server;

use strict;
use diagnostics;
use vars qw(@ISA);
use Net::Server::PreFork;

@ISA = qw(Net::Server::PreFork);


#
#  Modules we use.
#
use Xen::Monitor;
use Xen::Host;

#
#  The plugin callbacks
#
my @callbacks;

#
#  Start the server.
#
Xen::Monitor::Server->run();
exit;





=head2 loadAllPlugins

  Load all our plugins from the directory /usr/share/perl5/Xen/Monitor/Plugins

  If a plugin fails to compile, due to a syntax error, we will abort.

=cut

sub loadAllPlugins
{
    my $count = 0;

    #
    # Get all the possible plugin files.
    #
    my @plugins = sort( glob(  "/usr/share/perl5/Xen/Monitor/Plugins/*.pm" ) );
    
    #
    # Keep count of the plugins.
    #
    foreach my $plugin (@plugins)
    {
	eval { require( $plugin ); };
	
	if ( $@ )
	{
	    print $@;
	    exit;
	}
	$count += 1;
    }
    
    return( $count );
}




=head2  register_plugin

  Register a plugin in the global namespace.  

  This is called by each of the plugins loaded by the server, so that they
 may register their commands.

=cut

sub register_plugin 
{
    #
    #  Find the class name of the plugin which is registering itself,
    # and save it away so we can later use it.
    #
    my $class = shift;
    push @callbacks, $class;
}




=head2 pre_loop_hook

  This function is called by Net::Server before the server stats accepting
 connections.

  This is a perfect chance to load our plugins.

=cut

sub pre_loop_hook()
{
    my ($self) = ( @_ );

    #
    # Load all plugins, and display the number loaded.
    #
    my $count = loadAllPlugins();
    $self->log(2, "Plugins Loaded: $count\n" );

}




=head2 process_request

  This is the main routine of our server.  It is responsible for
 reading requests from clients.

  Each read line is assumed to be of the form:

    command arg arg2 ... argN

  Each loaded plugin is allowed the opportunity to handle the command,
 and unrecognised commands will result in an error.

=cut

sub process_request 
{
    #
    # This object.
    #
    my $self  = shift;

    #
    # Only wait 30 seconds for a command before timing out.
    # this is required to avoid timeouts.
    #
    my $timeout = 30; 

    eval 
    {
	local $SIG{ALRM} = sub { die "Timed Out!\n" };
	
	my $previous_alarm = alarm($timeout);
	while( <STDIN> )
	{
	    s/\r?\n$//;

	    my $line = $_;

	    #
	    #  Determine the command invoked, and any optional arguments.
	    #
	    if ( $line =~ /^([^ \t]+)[ \t]*(.*)/ )
	    {
		#
		#  The command and arguments from the host.
		#
		my $word = $1;
		my $args = $2;

		#
		# Did we find a plugin to handle the command?
		#
		my $handled = 0;

		foreach my $plugin (@callbacks) 
		{
		    #
		    # Create new plugin instance.
		    #
		    next if ( ! UNIVERSAL::can( $plugin, 'new' ) );
		    my $module = $plugin->new();

		    if ( UNIVERSAL::can($module, 'getCommand' ) )
		    {
			#
			# Find the commands this plugin will handle.
			#
			my %handled = $module->getCommand();

			#
			# If the command hasn't been handled, and
			# the plugin may handle it then we call it.
			#
			if ( $handled == 0 && $handled{lc $word} )
			{
			    #
			    #  We'll mark this request as recognised
			    # since the module will handle it.
			    #
			    $handled = 1;
			    my @arguments = split( / /, $args );

			    #
			    #  There are two types of plugins.
			    # 1.  "auth" - for authentication.
			    # 2.  "any"  - for running commands.
			    #
			    if ( UNIVERSAL::can($module, 'canAuth' ) )
			    {
				#
				#  Authenticate the user.
				#
				if ( $module->canAuth() )
				{
				    $self->{auth} = $module->runAuth( $self, $word, @arguments );

				    #
				    #  Authentication modules may optionally
				    # return a list of the instances the user
				    # may control.
				    #
				    #  This allows interesting customizations.
				    #
				    #
				    if ( UNIVERSAL::can($module, 'getHosts' ) )
				    {
					#
					# Find the instances this user
					# may control.
					#
					$self->{hosts} = $module->getHosts($self->{auth} );
				    }
				}
			    }
			    else
			    {
				#
				#  The module cannot run authentication 
 				# requests.  So just a command.
				#
				#  The plugin can handle it.  But does
				# this plugin require us to be authenticated?
				#
				#
				if ( $self->{auth} || ( $plugin->needsAuth() == 0 ) )
				{
				    $module->runCommand( $self, $word, @arguments );
				}
				else
				{
				    print "400 NOT AUTHENTICATED\r\n";
				}
			    }
			}
		    }
		}
		
		if ( ! $handled )
		{
		    print "400 UNRECOGNISED COMMAND '$word'\r\n";
		}
	    }
	    else 
	    {
		print "400 INVALID REQUEST \"$_\"\r\n";
	    }
	    alarm($timeout);
	}
	alarm($previous_alarm);

    };

    if( $@=~/timed out/i )
    {
	print STDOUT "Timed Out.\r\n";
	return;
    }

}





=head2 canControl

  Return 1 if the current authenticated user can control the given host.

=cut

sub canControl
{
    my ( $self, $host ) = ( @_ );

    my $user	 = $self->{auth};
    my $host_ref = $self->{hosts};
    my @hosts	 = @$host_ref;

    foreach my $h ( @hosts )
    {
	if ( $h eq "*" )
	{
	    return 1;
	}

	if ( lc($h) eq lc($host) )
	{
	    return 1;
	}
    }
    return 0;
} 



=head1 LICENSE

Copyright (c) 2005 by Steve Kemp.  All rights reserved.

This module is free software;
you can redistribute it and/or modify it under
the terms of the GNU General Public License.

=cut


1;
